#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <commctrl.h>
#include "AkelDLL.h"
#include "Resources\Resource.h"


//Defines
#define IT_DELIM_SKIP  1
#define IT_DELIM_ASTAB 2
#define IT_DELIM_ASIS  3

#define STRID_GLOBALHOTKEYS 1
#define STRID_DELIMSKIP     2
#define STRID_DELIMASTAB    3
#define STRID_DELIMASIS     4
#define STRID_OK            5
#define STRID_CANCEL        6


//Functions prototypes
BOOL CALLBACK SetupDlgProcA(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK SetupDlgProcW(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK NewMainProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
void WaitForReleaseVkKeys(DWORD dwThreadCurrent, DWORD dwThreadTarget, DWORD dwTimeout);
void StartHotkey();
void StopHotkey();
void ReadOptionsA();
void ReadOptionsW();
void SaveOptionsA();
void SaveOptionsW();
char* GetLangStringA(LANGID wLangID, int nStringID);
wchar_t* GetLangStringW(LANGID wLangID, int nStringID);
void InitCommon(PLUGINDATA *pd);
void InitMain();
void UninitMain();

//Global variables
char szPluginName[MAX_PATH];
wchar_t wszPluginName[MAX_PATH];
HINSTANCE hInstanceDLL;
HWND hMainWnd;
HWND hWndEdit;
HICON hMainIcon;
BOOL bOldWindows;
LANGID wLangSystem;
BOOL bInitCommon=FALSE;
BOOL bInitMain=FALSE;
int nHotkeyDelimSkipID=0;
int nHotkeyDelimAsTabID=0;
int nHotkeyDelimAsIsID=0;
DWORD dwHotkeyDelimSkip=MAKEWORD(VK_F9, MOD_CONTROL);
DWORD dwHotkeyDelimAsTab=MAKEWORD(VK_F10, MOD_CONTROL);
DWORD dwHotkeyDelimAsIs=MAKEWORD(VK_F11, MOD_CONTROL);
BOOL bEmulatePress=TRUE;
WNDPROCDATA *NewMainProcData=NULL;


//Identification
extern "C" void __declspec(dllexport) DllAkelPadID(PLUGINVERSION *pv)
{
  pv->dwAkelDllVersion=AKELDLL;
  pv->dwExeMinVersion3x=MAKE_IDENTIFIER(3, 7, 4, 0);
  pv->dwExeMinVersion4x=MAKE_IDENTIFIER(4, 4, 0, 0);
  pv->pPluginName="PasteSerial";
}

//Plugin extern function
extern "C" void __declspec(dllexport) Main(PLUGINDATA *pd)
{
  if (pd->lpbAutoLoad)
  {
    *pd->lpbAutoLoad=TRUE;
    return;
  }

  if (!bInitCommon) InitCommon(pd);

  //Is plugin already loaded?
  if (bInitMain)
  {
    UninitMain();
  }
  else
  {
    InitMain();

    //Stay in memory, and show as active
    pd->nUnload=UD_NONUNLOAD_ACTIVE;
  }
}

extern "C" void __declspec(dllexport) Settings(PLUGINDATA *pd)
{
  if (pd->lpbAutoLoad)
  {
    *pd->lpbAutoLoad=FALSE;
    return;
  }

  if (!bInitCommon) InitCommon(pd);

  if (bOldWindows)
    DialogBoxA(hInstanceDLL, MAKEINTRESOURCEA(IDD_SETUP), hMainWnd, (DLGPROC)SetupDlgProcA);
  else
    DialogBoxW(hInstanceDLL, MAKEINTRESOURCEW(IDD_SETUP), hMainWnd, (DLGPROC)SetupDlgProcW);

  //If plugin already loaded, stay in memory, but show as non-active
  if (pd->bInMemory) pd->nUnload=UD_NONUNLOAD_NONACTIVE;
}


BOOL CALLBACK SetupDlgProcA(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  static HWND hWndHotkeyDelimSkip;
  static HWND hWndHotkeyDelimAsTab;
  static HWND hWndHotkeyDelimAsIs;

  if (uMsg == WM_INITDIALOG)
  {
    SendMessage(hDlg, WM_SETICON, (WPARAM)ICON_BIG, (LPARAM)hMainIcon);
    hWndHotkeyDelimSkip=GetDlgItem(hDlg, IDC_HOTKEY_DELIMSKIP);
    hWndHotkeyDelimAsTab=GetDlgItem(hDlg, IDC_HOTKEY_DELIMASTAB);
    hWndHotkeyDelimAsIs=GetDlgItem(hDlg, IDC_HOTKEY_DELIMASIS);

    SetDlgItemTextA(hDlg, IDC_HOTKEY_GROUP, GetLangStringA(wLangSystem, STRID_GLOBALHOTKEYS));
    SetDlgItemTextA(hDlg, IDC_HOTKEY_DELIMSKIP_LABEL, GetLangStringA(wLangSystem, STRID_DELIMSKIP));
    SetDlgItemTextA(hDlg, IDC_HOTKEY_DELIMASTAB_LABEL, GetLangStringA(wLangSystem, STRID_DELIMASTAB));
    SetDlgItemTextA(hDlg, IDC_HOTKEY_DELIMASIS_LABEL, GetLangStringA(wLangSystem, STRID_DELIMASIS));
    SetDlgItemTextA(hDlg, IDOK, GetLangStringA(wLangSystem, STRID_OK));
    SetDlgItemTextA(hDlg, IDCANCEL, GetLangStringA(wLangSystem, STRID_CANCEL));

    SendMessage(hWndHotkeyDelimSkip, HKM_SETHOTKEY, dwHotkeyDelimSkip, 0);
    SendMessage(hWndHotkeyDelimAsTab, HKM_SETHOTKEY, dwHotkeyDelimAsTab, 0);
    SendMessage(hWndHotkeyDelimAsIs, HKM_SETHOTKEY, dwHotkeyDelimAsIs, 0);
  }
  else if (uMsg == WM_COMMAND)
  {
    if (LOWORD(wParam) == IDOK)
    {
      dwHotkeyDelimSkip=SendMessage(hWndHotkeyDelimSkip, HKM_GETHOTKEY, 0, 0);
      dwHotkeyDelimAsTab=SendMessage(hWndHotkeyDelimAsTab, HKM_GETHOTKEY, 0, 0);
      dwHotkeyDelimAsIs=SendMessage(hWndHotkeyDelimAsIs, HKM_GETHOTKEY, 0, 0);

      if (bInitMain)
      {
        UninitMain();
        InitMain();
      }
      SaveOptionsA();
      EndDialog(hDlg, 0);
      return TRUE;
    }
    else if (LOWORD(wParam) == IDCANCEL)
    {
      EndDialog(hDlg, 0);
      return TRUE;
    }
  }
  else if (uMsg == WM_CLOSE)
  {
    PostMessage(hDlg, WM_COMMAND, IDCANCEL, 0);
    return TRUE;
  }
  return FALSE;
}

BOOL CALLBACK SetupDlgProcW(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  static HWND hWndHotkeyDelimSkip;
  static HWND hWndHotkeyDelimAsTab;
  static HWND hWndHotkeyDelimAsIs;

  if (uMsg == WM_INITDIALOG)
  {
    SendMessage(hDlg, WM_SETICON, (WPARAM)ICON_BIG, (LPARAM)hMainIcon);
    hWndHotkeyDelimSkip=GetDlgItem(hDlg, IDC_HOTKEY_DELIMSKIP);
    hWndHotkeyDelimAsTab=GetDlgItem(hDlg, IDC_HOTKEY_DELIMASTAB);
    hWndHotkeyDelimAsIs=GetDlgItem(hDlg, IDC_HOTKEY_DELIMASIS);

    SetDlgItemTextW(hDlg, IDC_HOTKEY_GROUP, GetLangStringW(wLangSystem, STRID_GLOBALHOTKEYS));
    SetDlgItemTextW(hDlg, IDC_HOTKEY_DELIMSKIP_LABEL, GetLangStringW(wLangSystem, STRID_DELIMSKIP));
    SetDlgItemTextW(hDlg, IDC_HOTKEY_DELIMASTAB_LABEL, GetLangStringW(wLangSystem, STRID_DELIMASTAB));
    SetDlgItemTextW(hDlg, IDC_HOTKEY_DELIMASIS_LABEL, GetLangStringW(wLangSystem, STRID_DELIMASIS));
    SetDlgItemTextW(hDlg, IDOK, GetLangStringW(wLangSystem, STRID_OK));
    SetDlgItemTextW(hDlg, IDCANCEL, GetLangStringW(wLangSystem, STRID_CANCEL));

    SendMessage(hWndHotkeyDelimSkip, HKM_SETHOTKEY, dwHotkeyDelimSkip, 0);
    SendMessage(hWndHotkeyDelimAsTab, HKM_SETHOTKEY, dwHotkeyDelimAsTab, 0);
    SendMessage(hWndHotkeyDelimAsIs, HKM_SETHOTKEY, dwHotkeyDelimAsIs, 0);
  }
  else if (uMsg == WM_COMMAND)
  {
    if (LOWORD(wParam) == IDOK)
    {
      dwHotkeyDelimSkip=SendMessage(hWndHotkeyDelimSkip, HKM_GETHOTKEY, 0, 0);
      dwHotkeyDelimAsTab=SendMessage(hWndHotkeyDelimAsTab, HKM_GETHOTKEY, 0, 0);
      dwHotkeyDelimAsIs=SendMessage(hWndHotkeyDelimAsIs, HKM_GETHOTKEY, 0, 0);

      if (bInitMain)
      {
        UninitMain();
        InitMain();
      }
      SaveOptionsW();
      EndDialog(hDlg, 0);
      return TRUE;
    }
    else if (LOWORD(wParam) == IDCANCEL)
    {
      EndDialog(hDlg, 0);
      return TRUE;
    }
  }
  else if (uMsg == WM_CLOSE)
  {
    PostMessage(hDlg, WM_COMMAND, IDCANCEL, 0);
    return TRUE;
  }
  return FALSE;
}

LRESULT CALLBACK NewMainProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  if (uMsg == WM_HOTKEY)
  {
    char *szSerial=NULL;
    char *pSerial=NULL;
    HWND hWndTargetForeground;
    HWND hWndTargetFocus;
    DWORD dwTargetForeground;
    DWORD dwThreadCurrent;
    WORD wVk;
    int nInsertType=0;

    if (HIBYTE(dwHotkeyDelimSkip) == LOWORD(lParam) && LOBYTE(dwHotkeyDelimSkip) == HIWORD(lParam))
      nInsertType=IT_DELIM_SKIP;
    else if (HIBYTE(dwHotkeyDelimAsTab) == LOWORD(lParam) && LOBYTE(dwHotkeyDelimAsTab) == HIWORD(lParam))
      nInsertType=IT_DELIM_ASTAB;
    else if (HIBYTE(dwHotkeyDelimAsIs) == LOWORD(lParam) && LOBYTE(dwHotkeyDelimAsIs) == HIWORD(lParam))
      nInsertType=IT_DELIM_ASIS;

    if (nInsertType)
    {
      hWndTargetForeground=GetForegroundWindow();
      dwTargetForeground=GetWindowThreadProcessId(hWndTargetForeground, NULL);
      dwThreadCurrent=GetCurrentThreadId();

      if (dwThreadCurrent != dwTargetForeground)
      {
        if (AttachThreadInput(dwThreadCurrent, dwTargetForeground, TRUE))
        {
          WaitForReleaseVkKeys(dwThreadCurrent, dwTargetForeground, INFINITE);

          if (hWndTargetFocus=GetFocus())
          {
            //Get clipboard text
            {
              HGLOBAL hData;
              LPVOID pData;
              int nDataLen;

              if (OpenClipboard(NULL))
              {
                if (hData=GetClipboardData(CF_TEXT))
                {
                  if (pData=GlobalLock(hData))
                  {
                    nDataLen=lstrlenA((char *)pData) + 1;

                    if (szSerial=(char *)GlobalAlloc(GPTR, nDataLen))
                    {
                      lstrcpynA(szSerial, (char *)pData, nDataLen);
                    }
                    GlobalUnlock(hData);
                  }
                }
                CloseClipboard();
              }
            }

            //Post serial
            if (szSerial)
            {
              pSerial=szSerial;

              while (*pSerial && *pSerial != '\n')
              {
                if (nInsertType != IT_DELIM_ASIS)
                {
                  if (*pSerial == '-' || *pSerial == ' ' || *pSerial == '\t')
                  {
                    if (nInsertType == IT_DELIM_ASTAB)
                    {
                      if (GetFocus() == hWndTargetFocus)
                      {
                        if (bEmulatePress)
                        {
                          WaitForReleaseVkKeys(dwThreadCurrent, dwTargetForeground, INFINITE);
                          keybd_event(VK_TAB, 0, KEYEVENTF_EXTENDEDKEY, 0);
                          keybd_event(VK_TAB, 0, KEYEVENTF_EXTENDEDKEY|KEYEVENTF_KEYUP, 0);
                          WaitForReleaseVkKeys(dwThreadCurrent, dwTargetForeground, INFINITE);
                        }
                        else SendMessage(hWndTargetFocus, WM_CHAR, '\t', 1);
                      }
                    }
                    while (*++pSerial == '-' || *pSerial == ' ' || *pSerial == '\t');
                    hWndTargetFocus=GetFocus();
                    continue;
                  }
                }

                if (bEmulatePress)
                {
                  wVk=VkKeyScan(*pSerial);

                  WaitForReleaseVkKeys(dwThreadCurrent, dwTargetForeground, INFINITE);
                  if (HIBYTE(wVk) & 1) keybd_event(VK_SHIFT, 0, KEYEVENTF_EXTENDEDKEY, 0);
                  if (HIBYTE(wVk) & 2) keybd_event(VK_CONTROL, 0, KEYEVENTF_EXTENDEDKEY, 0);
                  if (HIBYTE(wVk) & 4) keybd_event(VK_MENU, 0, KEYEVENTF_EXTENDEDKEY, 0);
                  keybd_event(LOBYTE(wVk), 0, 0, 0);
                  keybd_event(LOBYTE(wVk), 0, KEYEVENTF_KEYUP, 0);
                  if (HIBYTE(wVk) & 4) keybd_event(VK_MENU, 0, KEYEVENTF_EXTENDEDKEY|KEYEVENTF_KEYUP, 0);
                  if (HIBYTE(wVk) & 2) keybd_event(VK_CONTROL, 0, KEYEVENTF_EXTENDEDKEY|KEYEVENTF_KEYUP, 0);
                  if (HIBYTE(wVk) & 1) keybd_event(VK_SHIFT, 0, KEYEVENTF_EXTENDEDKEY|KEYEVENTF_KEYUP, 0);
                  WaitForReleaseVkKeys(dwThreadCurrent, dwTargetForeground, INFINITE);
                }
                else SendMessage(hWndTargetFocus, WM_CHAR, *pSerial, 1);

                ++pSerial;
              }
              GlobalFree((HGLOBAL)szSerial);
              szSerial=NULL;
            }
          }
          AttachThreadInput(dwThreadCurrent, dwTargetForeground, FALSE);
        }
      }
    }
  }

  //Call next procedure
  return NewMainProcData->NextProc(hWnd, uMsg, wParam, lParam);
}


void WaitForReleaseVkKeys(DWORD dwThreadCurrent, DWORD dwThreadTarget, DWORD dwTimeout)
{
  DWORD dwStart=GetTickCount();
  BYTE lpKeyState[256];
  int i;

  Loop:
  if (dwTimeout == INFINITE || GetTickCount() - dwStart < dwTimeout)
  {
    AttachThreadInput(dwThreadCurrent, dwThreadTarget, FALSE);

    if (AttachThreadInput(dwThreadCurrent, dwThreadTarget, TRUE))
    {
      Sleep(0);

      if (GetKeyboardState(lpKeyState))
      {
        for (i=0; i < 256; ++i)
        {
          if (lpKeyState[i] & 0x80)
            goto Loop;
        }
      }
    }
  }
}


//// Options

void ReadOptionsA()
{
  PLUGINOPTIONA po;
  HANDLE hOptions;

  if (hOptions=(HANDLE)SendMessage(hMainWnd, AKD_BEGINOPTIONS, POB_READ, (LPARAM)szPluginName))
  {
    po.pOptionName="HotkeyDelimSkip";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&dwHotkeyDelimSkip;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="HotkeyDelimAsTab";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&dwHotkeyDelimAsTab;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="HotkeyDelimAsIs";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&dwHotkeyDelimAsIs;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    SendMessage(hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
  }
}

void ReadOptionsW()
{
  PLUGINOPTIONW po;
  HANDLE hOptions;

  if (hOptions=(HANDLE)SendMessage(hMainWnd, AKD_BEGINOPTIONS, POB_READ, (LPARAM)wszPluginName))
  {
    po.pOptionName=L"HotkeyDelimSkip";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&dwHotkeyDelimSkip;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"HotkeyDelimAsTab";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&dwHotkeyDelimAsTab;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"HotkeyDelimAsIs";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&dwHotkeyDelimAsIs;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    SendMessage(hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
  }
}

void SaveOptionsA()
{
  PLUGINOPTIONA po;
  HANDLE hOptions;

  if (hOptions=(HANDLE)SendMessage(hMainWnd, AKD_BEGINOPTIONS, POB_SAVE, (LPARAM)szPluginName))
  {
    po.pOptionName="HotkeyDelimSkip";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&dwHotkeyDelimSkip;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="HotkeyDelimAsTab";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&dwHotkeyDelimAsTab;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="HotkeyDelimAsIs";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&dwHotkeyDelimAsIs;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    SendMessage(hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
  }
}

void SaveOptionsW()
{
  PLUGINOPTIONW po;
  HANDLE hOptions;

  if (hOptions=(HANDLE)SendMessage(hMainWnd, AKD_BEGINOPTIONS, POB_SAVE, (LPARAM)wszPluginName))
  {
    po.pOptionName=L"HotkeyDelimSkip";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&dwHotkeyDelimSkip;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"HotkeyDelimAsTab";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&dwHotkeyDelimAsTab;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"HotkeyDelimAsIs";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&dwHotkeyDelimAsIs;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    SendMessage(hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
  }
}

char* GetLangStringA(LANGID wLangID, int nStringID)
{
  static char szStringBuf[MAX_PATH];

  WideCharToMultiByte(CP_ACP, 0, GetLangStringW(wLangID, nStringID), -1, szStringBuf, MAX_PATH, NULL, NULL);
  return szStringBuf;
}

wchar_t* GetLangStringW(LANGID wLangID, int nStringID)
{
  if (wLangID == LANG_RUSSIAN)
  {
    if (nStringID == STRID_GLOBALHOTKEYS)
      return L"\x0413\x043B\x043E\x0431\x0430\x043B\x044C\x043D\x044B\x0435\x0020\x0433\x043E\x0440\x044F\x0447\x0438\x0435\x0020\x043A\x043B\x0430\x0432\x0438\x0448\x0438";
    if (nStringID == STRID_DELIMSKIP)
      return L"\x041F\x0440\x043E\x043F\x0443\x0441\x043A\x0430\x0442\x044C\x0020\x0440\x0430\x0437\x0434\x0435\x043B\x0438\x0442\x0435\x043B\x0438";
    if (nStringID == STRID_DELIMASTAB)
      return L"\x0417\x0430\x043C\x0435\x043D\x044F\x0442\x044C\x0020\x0440\x0430\x0437\x0434\x0435\x043B\x0438\x0442\x0435\x043B\x0438\x0020\x0442\x0430\x0431\x043E\x043C";
    if (nStringID == STRID_DELIMASIS)
      return L"\x0412\x0441\x0442\x0430\x0432\x0438\x0442\x044C\x0020\x043A\x0430\x043A\x0020\x0435\x0441\x0442\x044C";
    if (nStringID == STRID_OK)
      return L"\x004F\x004B";
    if (nStringID == STRID_CANCEL)
      return L"\x041E\x0442\x043C\x0435\x043D\x0430";
  }
  else
  {
    if (nStringID == STRID_GLOBALHOTKEYS)
      return L"Global hotkeys";
    if (nStringID == STRID_DELIMSKIP)
      return L"Skip delimiters";
    if (nStringID == STRID_DELIMASTAB)
      return L"Replace delimiter with tab";
    if (nStringID == STRID_DELIMASIS)
      return L"Paste as is";
    if (nStringID == STRID_OK)
      return L"OK";
    if (nStringID == STRID_CANCEL)
      return L"Cancel";
  }
  return L"";
}

void InitCommon(PLUGINDATA *pd)
{
  bInitCommon=TRUE;
  hInstanceDLL=pd->hInstanceDLL;
  hMainWnd=pd->hMainWnd;
  hMainIcon=pd->hMainIcon;
  bOldWindows=pd->bOldWindows;
  wLangSystem=PRIMARYLANGID(pd->wLangSystem);

  if (bOldWindows)
  {
    int i;

    for (i=0; ((char *)pd->pFunction)[i] != ':'; ++i)
      szPluginName[i]=((char *)pd->pFunction)[i];
    szPluginName[i]='\0';
  }
  else
  {
    int i;

    for (i=0; ((wchar_t *)pd->pFunction)[i] != ':'; ++i)
      wszPluginName[i]=((wchar_t *)pd->pFunction)[i];
    wszPluginName[i]='\0';
  }

  if (bOldWindows)
    ReadOptionsA();
  else
    ReadOptionsW();
}

void InitMain()
{
  bInitMain=TRUE;

  if (nHotkeyDelimSkipID=GlobalAddAtomA("AkelPad::PasteSerial::DelimSkip"))
    RegisterHotKey(hMainWnd, nHotkeyDelimSkipID, HIBYTE(dwHotkeyDelimSkip), LOBYTE(dwHotkeyDelimSkip));
  if (nHotkeyDelimAsTabID=GlobalAddAtomA("AkelPad::PasteSerial::DelimAsTab"))
    RegisterHotKey(hMainWnd, nHotkeyDelimAsTabID, HIBYTE(dwHotkeyDelimAsTab), LOBYTE(dwHotkeyDelimAsTab));
  if (nHotkeyDelimAsIsID=GlobalAddAtomA("AkelPad::PasteSerial::DelimAsIs"))
    RegisterHotKey(hMainWnd, nHotkeyDelimAsIsID, HIBYTE(dwHotkeyDelimAsIs), LOBYTE(dwHotkeyDelimAsIs));

  //SubClass main procedure
  NewMainProcData=NULL;
  SendMessage(hMainWnd, AKD_SETMAINPROC, (WPARAM)NewMainProc, (LPARAM)&NewMainProcData);
}

void UninitMain()
{
  bInitMain=FALSE;

  //Remove subclass
  if (NewMainProcData)
  {
    SendMessage(hMainWnd, AKD_SETMAINPROC, (WPARAM)NULL, (LPARAM)&NewMainProcData);
    NewMainProcData=NULL;
  }

  if (nHotkeyDelimSkipID)
  {
    if (UnregisterHotKey(hMainWnd, nHotkeyDelimSkipID))
    {
      GlobalDeleteAtom(nHotkeyDelimSkipID);
      nHotkeyDelimSkipID=0;
    }
  }
  if (nHotkeyDelimAsTabID)
  {
    if (UnregisterHotKey(hMainWnd, nHotkeyDelimAsTabID))
    {
      GlobalDeleteAtom(nHotkeyDelimAsTabID);
      nHotkeyDelimAsTabID=0;
    }
  }
  if (nHotkeyDelimAsIsID)
  {
    if (UnregisterHotKey(hMainWnd, nHotkeyDelimAsIsID))
    {
      GlobalDeleteAtom(nHotkeyDelimAsIsID);
      nHotkeyDelimAsIsID=0;
    }
  }
}

//Entry point
extern "C" BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
  if (fdwReason == DLL_PROCESS_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_DETACH)
  {
  }
  else if (fdwReason == DLL_PROCESS_DETACH)
  {
    if (bInitMain) UninitMain();
  }
  return TRUE;
}
